@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-__version__ = "1.1.0"
+__version__ = "1.3.0"
 
 
 import asyncio
@@ -8,6 +8,7 @@ import logging
 from collections.abc import Callable
 from typing import Any
 
+import async_timeout
 from bleak import BleakClient, BleakError
 from bleak.backends.device import BLEDevice
 
@@ -25,7 +26,13 @@ MAX_TRANSIENT_ERRORS = 9
 # to finish in < 60s or declare we cannot connect
 
 MAX_CONNECT_ATTEMPTS = 5
-BLEAK_TIMEOUT = 10
+BLEAK_TIMEOUT = 12
+
+# Bleak may not always timeout
+# since the dbus connection can stall
+# so we have an additional timeout to
+# be sure we do not block forever
+BLEAK_SAFETY_TIMEOUT = 13
 
 # These errors are transient with dbus, and we should retry
 TRANSIENT_ERRORS = {"le-connection-abort-by-local", "br-connection-canceled"}
@@ -74,13 +81,30 @@ async def establish_connection(
         attempt += 1
         _LOGGER.debug("%s: Connecting (attempt: %s)", name, attempt)
         try:
-            await client.connect(timeout=BLEAK_TIMEOUT)
+            async with async_timeout.timeout(BLEAK_SAFETY_TIMEOUT):
+                await client.connect(timeout=BLEAK_TIMEOUT)
         except asyncio.TimeoutError as exc:
             timeouts += 1
             _LOGGER.debug(
                 "%s: Timed out trying to connect (attempt: %s)", name, attempt
             )
             _raise_if_needed(name, exc)
+        except BrokenPipeError as exc:
+            # BrokenPipeError is raised by dbus-next when the device disconnects
+            #
+            # bleak.exc.BleakDBusError: [org.bluez.Error] le-connection-abort-by-local
+            # During handling of the above exception, another exception occurred:
+            # Traceback (most recent call last):
+            # File "bleak/backends/bluezdbus/client.py", line 177, in connect
+            #   reply = await self._bus.call(
+            # File "dbus_next/aio/message_bus.py", line 63, in write_callback
+            #   self.offset += self.sock.send(self.buf[self.offset:])
+            # BrokenPipeError: [Errno 32] Broken pipe
+            transient_errors += 1
+            _LOGGER.debug(
+                "%s: Failed to connect: %s (attempt: %s)", name, str(exc), attempt
+            )
+            _raise_if_needed(name, exc)
         except BLEAK_EXCEPTIONS as exc:
             bleak_error = str(exc)
             if any(error in bleak_error for error in TRANSIENT_ERRORS):
